/*
 * SPDX-FileCopyrightText: Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef NVIDIA_GXF_SERIALIZATION_TESTS_SERIALIZATION_TESTER_HPP_
#define NVIDIA_GXF_SERIALIZATION_TESTS_SERIALIZATION_TESTER_HPP_

#include <memory>

#include "gxf/serialization/entity_serializer.hpp"
#include "gxf/serialization/serialization_buffer.hpp"
#include "gxf/std/codelet.hpp"
#include "gxf/std/receiver.hpp"
#include "gxf/std/transmitter.hpp"

namespace nvidia {
namespace gxf {
namespace test {

// Codelet that serializes incoming messages and stores them in a buffer
// Messages are immediately deserialized from the buffer and published
class SerializationTester : public Codelet {
 public:
  gxf_result_t registerInterface(Registrar* registrar) override;
  gxf_result_t initialize() override { return GXF_SUCCESS; }
  gxf_result_t deinitialize() override { return GXF_SUCCESS; }
  gxf_result_t tick() override;

 private:
  Parameter<Handle<Receiver>> input_;
  Parameter<Handle<Transmitter>> output_;
  Parameter<Handle<EntitySerializer>> entity_serializer_;
  Parameter<Handle<SerializationBuffer>> serialization_buffer_;
};

}  // namespace test
}  // namespace gxf
}  // namespace nvidia

#endif  // NVIDIA_GXF_SERIALIZATION_TESTS_SERIALIZATION_TESTER_HPP_
