/*
 * SPDX-FileCopyrightText: Copyright (c) 2023-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef HOLOSCAN_OPERATORS_PING_RX_HPP
#define HOLOSCAN_OPERATORS_PING_RX_HPP

#include <holoscan/holoscan.hpp>

namespace holoscan::ops {

/**
 * @brief Simple receiver operator.
 *
 * This is an example of a native operator with one input port.
 * On each tick, it receives an integer from the "in" port.
 *
 * ==Named Inputs==
 *
 * - **in** : any
 *   - A received value.
 */
class PingRxOp : public Operator {
 public:
  HOLOSCAN_OPERATOR_FORWARD_ARGS(PingRxOp)

  PingRxOp() = default;

  void setup(OperatorSpec& spec) override;

  void compute(InputContext& op_input, OutputContext&, ExecutionContext&) override;
};

}  // namespace holoscan::ops

#endif /* HOLOSCAN_OPERATORS_PING_RX_HPP */
